@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2015 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2016 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -116,6 +116,13 @@ bool RenderTexture::isRepeated() const
 }
 
 
+////////////////////////////////////////////////////////////
+bool RenderTexture::generateMipmap()
+{
+    return m_texture.generateMipmap();
+}
+
+
 ////////////////////////////////////////////////////////////
 bool RenderTexture::setActive(bool active)
 {
@@ -131,6 +138,7 @@ void RenderTexture::display()
     {
         m_impl->updateTexture(m_texture.m_texture);
         m_texture.m_pixelsFlipped = true;
+        m_texture.invalidateMipmap();
     }
 }
 
